function [] = ParamGenTFFDASparseL(M,N,R,L,eps)
% Parameter generation for a synthetic operator recovery
% The operators are approximately UNTF.
% The initial operator is drifted from the original operator

% M = Signal space dimension
% N = Analysis space dimension
% R = Cosparsity
% L = Population of the training samples
% eps = Deviation of the starting operator from desired operator

% Constrained Analysis Operator Learning for Cosparse Signal Modelling
% Written by Mehrdad Yaghoobi, version 1.0                                    
%     
% Copyright 2013 Mehrdad Yaghoobi, Sangnam Nam, Remi Gribonval and Mike E. Davies
% 
% For all details please refer to README.m
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose. 

% Original operator generation

Dorig = randn(M,N);
for i = 1:100,
    Dorig = ProjTF(Dorig);
    Dorig = ProjUNColBall(Dorig);
end

% Initial operator generation

D = randn(M,N);
%%%%%
D = ProjUNColBall(D);
D = Dorig + eps*D;
%%%%%
for i = 1:100,
    D = ProjTF(D);
    D = ProjUNColBall(D);
end

% Training data generation

Y = zeros(M,L);
for l = 1:L
    lambda = randperm(N);
    lambda = lambda(1:R);
    [U,S,V] = svd(Dorig(:,lambda));
    Null = U(:,R+1:M);
    Y(:,l) = Null*randn(M-R,1);
end

%%% Saving the parameters

if ~(exist('Parameters','dir'))
    mkdir Parameters
end
cd Parameters
save(['ParamTFFDA',num2str(R),'X',num2str(M),'X',num2str(N),'X',num2str(L),'X',num2str(eps),'kAFD2X01.mat'],'D','Dorig','Y')
cd ..