function [] = SubGPAOR(M,N,R,L,eps,T)
% Projected subgardient algorithm for analysis operator recovery
% Some experiments to recover synthetic analysis operators using fixed cospaarsity training samples

% Initialisation with a deviated operator, from the desired one, to check the
% basin of atraction of a fixed point and local optimality of the objective with a given
% operator

% The simulation is repeating T times with the same problem setting, but new
% operators and training samples each time.


% M = Signal space dimension
% N = Analysis space dimension
% R = Cosparsity
% L = Population of the training samples
% eps = Deviation of the starting operator from desired operator
% T = Number of trials

% Constrained Analysis Operator Learning for Cosparse Signal Modelling
% Written by Mehrdad Yaghoobi, version 1.0                                    
%     
% Copyright 2013 Mehrdad Yaghoobi, Sangnam Nam, Remi Gribonval and Mike E. Davies
% 
% For all details please refer to README.m
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose. 

IT = 50000; % Number of Iterations
for t = 1:T,
    ParamGenTFFDASparseL(M,N,R,L,eps) % Generating operators and training samples
    cd Parameters
    load(['ParamTFFDA',num2str(R),'X',num2str(M),'X',num2str(N),'X',num2str(L),'X',num2str(eps),'kAFD2X01.mat'])
    cd ..
    Do = D;
    [M,N] = size(Do);
    [M,L] = size(Y);
    alpha = .5*10^-5; % Gradient Step Size 
    
    %%%%% Projected Sub Gradient 
    
    cost(1) = sum(sum(abs(Y'*D))); % Objective value
    for i = 1 : IT
        SG = Y*signrand(Y'*D);             
        D = D - alpha*SG;   
        D = ProjTF(D);
        D = ProjUNColBall(D);
        c = sum(sum(abs(Y'*D)));
        cost(i+1) = c; 
    end
    Psi = Dorig;
    Dn = ProjUNColBall(D);
    Psin = ProjUNColBall(Psi);
    err(t) = 100*sum((size(D,1)/size(D,2))*ones(1,length(D)) - max(abs(Dorig'*D)) < 0.001)/N;
end

%%% Saving the solution

if ~(exist('Solutions','dir'))
    mkdir Solutions
end
cd Solutions
save(['AOR',num2str(R),'X',num2str(M),'X',num2str(N),'X',num2str(L),'X',num2str(eps),'X',num2str(T),'v01.mat'],'err','cost')
cd ..