% Face Image Denoising using Analysis Operator Learning

% Yale Face Image Database (Cropped Version) should be in the Matlab path
% The data base can be obtained from:
% http://vision.ucsd.edu/extyaleb/CroppedYaleBZip/CroppedYale.zip

% Constrained Analysis Operator Learning for Cosparse Signal Modelling
% Written by Mehrdad Yaghoobi, version 1.0                                    
%     
% Copyright 2013 Mehrdad Yaghoobi, Sangnam Nam, Remi Gribonval and Mike E. Davies
% 
% For all details please refer to README.m
%
% This software is a free software distributed under the terms of the GNU 
% Public License version 3 (http://www.gnu.org/licenses/gpl.txt). You can 
% redistribute it and/or modify it under the terms of this licence, for 
% personal and non-commercial use and research purpose. 

M = 64; % Signal Dimensiona (8 by 8 image patches)
N = 128; % Analysis Space Dimension
L = 128*N; % Number of Training Samples
Lambda = {0.3,0.1}; % Lagrange Multiplier in the Objective Function
ParamGenFace(M,N,L)

% Noise Aware Analysis Operator Learning

for i=1:length(Lambda)
    NoisyAOLFaceRandLine(M,N,L,Lambda{i})
end

% Denoising a different face image

% The blocking effect is caused by not considering the overlapped image
% patches.

lab = ['(a)';'(b)';'(c)';'(d)';'(e)';'(f)';'(g)';'(h)'];

A = imread('yaleB15_P00A-005E-10.pgm');
AN = double(A) + 0.1*double(128*randn(size(A)));
ii = 1;
figure(1)
clf
subplot(1+length(Lambda),2,ii)
colormap(gray(256))
imagesc(A);
set(gca,'XTick',[],'YTick',[])
title(lab(ii,:))
ii = ii+1;
subplot(1+length(Lambda),2,ii)
imagesc(AN);
set(gca,'XTick',[],'YTick',[])
title(lab(ii,:))
ii=ii+1;
[H,W] = size(A);


for l = 1:length(Lambda)
    lambda = Lambda{l};
    load(['FaceNoisyAOLLine',num2str(M),'X',num2str(N),'X',num2str(L),'X',num2str(lambda),'.mat'])
    clear Y

    Ms = sqrt(M);
    Hn = H/Ms;
    Wn = W/Ms;
    
    Y = zeros(M,H*W/M);
    for h = 1:Hn
        for w = 1:Wn
            Y(:,Wn*(h-1)+w) = reshape(AN(Ms*(h-1)+1:Ms*(h),Ms*(w-1)+1:Ms*(w)),M,1);
        end    
    end
    
    Ym = mean(Y);
    Y = double(Y) - ones(M,1)*Ym;
    
    lambda2 = lambda;
    
    % Using a learned operator
    [X,err] = AnalysisL1Denoising(Y,Omega,lambda2);
    B = zeros(size(A));
    for h = 1:Hn
        for w = 1:Wn
            B(Ms*(h-1)+1:Ms*(h),Ms*(w-1)+1:Ms*(w)) = reshape(X(:,Wn*(h-1)+w)+ones(M,1)*Ym(Wn*(h-1)+w),Ms,Ms);
        end    
    end
    subplot(1+length(Lambda),2,ii)    
    imagesc(B);
    set(gca,'XTick',[],'YTick',[])
    title(lab(ii,:))
    ii = ii + 1;
    
    % Usinf the TV operator
    
    Btv = zeros(size(A));
    Omegatv = zeros(N,M); 
    I = eye(M);
    for i = 1:M, 
        I2h = reshape(I(:,i),Ms,Ms);
        I2r = [zeros(Ms,1),I2h(:,1:Ms-1)];
        I2v = reshape(I(i,:),Ms,Ms);
        I2d = [zeros(1,Ms);I2v(1:Ms-1,:)];
        Omegatv(i,:) = reshape(I2h-I2r,1,M);
        Omegatv(i+M,:) = reshape(I2v-I2d,1,M);
    end
    Omegatv = (ProjUNColBall(Omegatv'))'; % TV operator
    
    [Xtv,err] = AnalysisL1Denoising(Y,Omegatv,lambda2);

    for h = 1:Hn
        for w = 1:Wn
            Btv(Ms*(h-1)+1:Ms*(h),Ms*(w-1)+1:Ms*(w)) = reshape(Xtv(:,Wn*(h-1)+w)+ones(M,1)*Ym(Wn*(h-1)+w),Ms,Ms);
        end    
    end
    subplot(1+length(Lambda),2,ii)    
    imagesc(Btv)
    set(gca,'XTick',[],'YTick',[])
    title(lab(ii,:))
    ii = ii+1;
end
tresh = .99;
Zl0 = sum(abs(Omega*X)<=tresh);
Zl0tv = sum(abs(Omegatv*Xtv)<=tresh);
figure(2)
clf
subplot(311)
plot(Zl0,'k')
grid on
title('Cosparsity with the learned operator')
ax = axis;
axis([1,length(Y),ax(3),ax(4)])

subplot(312)
plot(Zl0tv,'k')
grid on
title('Cosparsity with the Total Variation operator')
axis([1,length(Y),ax(3),ax(4)])

subplot(313)
plot(Zl0-Zl0tv,'k')
grid on
hold on
plot(ones(length(Y),1)*mean(Zl0-Zl0tv),'k')
title('Differences between cosparsities')
xlabel('Image patch number')
ax = axis;
axis([1,length(Y),-20,40])